<?php

namespace GDPRGeoPopup;

use GDPRGeoPopup\database\DB;
use GDPRGeoPopup\src\AdminAPI;
use GDPRGeoPopup\src\AdminConfigPage;
use GDPRGeoPopup\src\AdminUsersPage;
use GDPRGeoPopup\src\FrontendPopup;

class Plugin
{
    /**
     * @var PluginOptions
     */
    protected $options;
    /**
     * @var DB;
     */
    protected $db;

    public function __construct()
    {
        $this->options = new PluginOptions();
        $this->db = new DB($this->options);
    }

    public function run()
    {
        $page1 = new AdminConfigPage($this->options, $this->db);
        $page1->init();
        $page2 = new AdminUsersPage($this->options, $this->db);
        $page2->init();

        $fp = new FrontendPopup($this->options, $this->db);
        $fp->init();

        $api = new AdminAPI($this->options, $this->db);
        $api->init();
    }

    public function activate()
    {
        $installedVersion = get_option( "gdpr_geo_popup" );

        if ($installedVersion)
            return;

        $this->db->createPopupsTable();
        $this->db->createPopupPagesTable();
        $this->db->createUsersTable();

        $this->db->insertSamplePopup();

        add_option( "gdpr_geo_popup", $this->options->version );
    }

    public function deactivate()
    {

    }
}