<?php

namespace GDPRGeoPopup\database;

use GDPRGeoPopup\PluginOptions;

class DB
{
    /**
     * @var PluginOptions
     */
    protected $pluginOptions;

    public function __construct($pluginOptions)
    {
        $this->pluginOptions = $pluginOptions;
    }

    public function createPopupsTable()
    {
        global $wpdb;
        $table_name = $wpdb->prefix . $this->pluginOptions->popupsTable;

        $sql = "CREATE TABLE IF NOT EXISTS  $table_name (
            id mediumint(9) NOT NULL AUTO_INCREMENT,
            title VARCHAR(1000) NOT NULL,
            internal_name  VARCHAR(1000),
            description TEXT,
            terms TEXT,
            text_color VARCHAR (255) DEFAULT '#000000' NOT NULL,
            color VARCHAR (255) DEFAULT '#000000' NOT NULL,
            button_color VARCHAR (255) DEFAULT '#FFFFFF' NOT NULL,
            button_text_color VARCHAR (255) DEFAULT '#000000' NOT NULL,
            submit_button_text VARCHAR (255) DEFAULT 'Submit' NOT NULL,
            cancel_button_text VARCHAR (255) DEFAULT 'Cancel' NOT NULL,
            countries VARCHAR (5000),
            redirect_link VARCHAR(1000),
            preference_link VARCHAR(1000) NULL,
            pp_link VARCHAR(1000) NULL,
            email VARCHAR (255) NULL,
            is_required TINYINT (1) DEFAULT 1 NOT NULL,
            is_deleted TINYINT (1) DEFAULT 0 NOT NULL,
            PRIMARY KEY  (id)
	    );";

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }

    public function createPopupPagesTable()
    {
        global $wpdb;
        $table_name = $wpdb->prefix . $this->pluginOptions->popupPagesTable;

        $sql = "CREATE TABLE IF NOT EXISTS  $table_name (
            id mediumint(9) NOT NULL AUTO_INCREMENT,
            popup_id mediumint(9) NOT NULL,
            page_id mediumint(9) NOT NULL,
            PRIMARY KEY  (id),
            INDEX `popup_id_index` (`popup_id`),
            INDEX `page_id_index` (`page_id`)
	    );";

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }

    public function createUsersTable()
    {
        global $wpdb;
        $table_name = $wpdb->prefix . $this->pluginOptions->usersTable;

        $sql = "CREATE TABLE IF NOT EXISTS $table_name (
            id mediumint(9) NOT NULL AUTO_INCREMENT,
            popup_id mediumint(9) NOT NULL,
            first_name VARCHAR(255) NOT NULL,
            last_name VARCHAR(255) NOT NULL,
            email VARCHAR (255) NOT NULL,
            consent TINYINT (1) DEFAULT 0 NOT NULL,
            time TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
            PRIMARY KEY  (id),
            INDEX `popup_id_index` (`popup_id`)
	    );";

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }

    public function insertSamplePopup()
    {
        global $wpdb;

        $table_name = $wpdb->prefix . $this->pluginOptions->popupsTable;
        $wpdb->insert($table_name, [
            'title'             => 'WAIT!',
            'internal_name'     => 'GDPR popup',
            'description'       => "It looks like you live in %COUNTRY%. To continue you'll need to give us explicit permission to follow up with you. Please submit your details below.",
            'terms'             => "[Company] TERMS AND CONDITIONS\nLast Updated: [DATE]\n\nINTRODUCTION\n\nThe Terms and Conditions (“Terms”) describe how [Insert full registered company name + address] (“Company,” “we,” and “our”) regulates your use of this website [insert your website] (the “Site”). Please read the following information carefully to understand our practices regarding your use of the Site. The Company may change the Terms at any time. The Company may inform you of the changes to the Terms using the available means of communication. The Company recommends you to check the Site frequently to see the actual version of the Terms and their previous versions.\n\nIf you represent a legal entity, you certify that you entitled by such a legal entity to conclude the Terms as the legal entity you represent.\n\n    1. PRIVACY POLICY \nOur Privacy Policy is available on a separate page. Our Privacy Policy explains to you how we process information about you. You shall understand that through your use of the Site you acknowledge the processing of this information shall be undertaken in accordance with the Privacy Policy. \n    2. [YOUR ACCOUNT] [Note: this applies only if a user can register on the Site]\n[When using the Site, you shall be responsible for ensuring the confidentiality of your account, password and other credentials and for secure access to your device. You shall not assign your account to anyone. The Company is not responsible for unauthorized access to your account that results from misappropriation or theft of your account. The Company may refuse or cancel service, terminate your account, and remove or edit content.\nThe Company does not knowingly collect personal data from persons under the age of 16 (sixteen). If you are under 16 (sixteen) years old, you may not use the Site and may not enter into the Terms under any circumstances.] [Note: this applies only if a user can register on the Site]\n    3. SERVICES\nThe Site allows you to use Services available on the Site. You shall not use the services for the illegal aims.\nWe may, at our sole discretion, set fees for using the Site for you. All prices are published separately on relevant pages on the Site. We may, at our sole discretion, at any time change any fees.\nWe may use certified payment systems which also may have their commissions. Such commissions may be implied on you when you choose a particular payment system. Detailed information about commissions of such payment systems may be found on their websites..\n    4. THIRD PARTY SERVICES \nThe Site may include links to other sites, applications, and platforms (hereinafter the \"Linked Sites\").\nThe Company does not control the Linked Sites, and shall not be responsible for the content and other materials of the Linked Sites. The Company makes these links available to you for providing the functionality or services on the Site.\n    5. PROHIBITED USES AND INTELLECTUAL PROPERTY\nThe Company grants you a non-transferable, non-exclusive, revocable license to access and use the Site from one device in accordance with the Terms.\nYou shall not use the Site for unlawful or prohibited purpose. You may not use the Site in a way that may disable, damage, or interfere in the Site.\nAll content present on the Site includes text, code, graphics, logos, images, compilation, software used on the Site (hereinafter and hereinbefore the \"Content\"). The Content is the property of the Company or its contractors and protected by intellectual property laws that protect such rights. You agree to use all copyright and other proprietary notices or restrictions contained in the Content and you are prohibited from changing the Content.\nYou may not publish, transmit, modify, reverse engineer, participate in the transfer, or create and sell derivative works, or in any way use any of the Content. Your enjoyment of the Site shall not entitle you to make any illegal and disallowed use of the Content, and in particular you shall not change proprietary rights or notices in the Content. You shall use the Content only for your personal and non-commercial use. The Company does not grant you any licenses to the intellectual property of the Company.\n    6. THE COMPANY MATERIALS\nBy posting, uploading, inputting, providing or submitting your Content you are granting the Company to use your Content in connection with the operation of Company's business including, but not limited to, the rights to transmit, publicly display, distribute, publicly perform, copy, reproduce, and translate your Content; and to publish your name in connection with your Content.\nNo compensation shall be paid with regard to the use of your Content. The Company shall have no obligation to publish or enjoy any Content you may send us and may remove your Content at any time.\nBy posting, uploading, inputting, providing or submitting your Content you warrant and represent that you own all of the rights to your Content. \n    7. DISCLAIMER OF CERTAIN LIABILITIES\nThe information available via the Site may include typographical errors or inaccuracies. The Company shall not be liable for these inaccuracies and errors.\nThe Company makes no representations about the availability, accuracy, reliability, suitability, and timeliness of the Content contained on and services available on the Site. To the maximum extent allowed by the applicable law, all such Content and services are provided on the \"as is\" basis. The Company disclaims all warranties and conditions regarding this Content and services, including warranties and provisions of merchantability, fitness for a certain purpose.\nTo the maximum extent permitted by the applicable law, in no event shall the Company be liable for any direct, indirect, incidental, consequential, special, punitive damages including, but not limited to, damages for loss of enjoyment, data or profits, in the connection with the enjoyment or execution of the Site in the context of the inability or delay to enjoy the Site or its services, or for any Content of the Site, or otherwise arising out of the enjoyment of the Site, based on contract and non-contract liability or other reason.\nIf the exclusion or limitation of liability for damages, whether consequential or incidental, are prohibited in a particular case, the exclusion or limitation of liability shall not apply to you.\n    8. INDEMNIFICATION\nYou agree to indemnify, defend and hold harmless the Company, its managers, directors, employees, agents, and third parties, for any costs, losses, expenses (including attorneys' fees), liabilities regarding or arising out of your enjoyment of or inability to enjoy the Site or its services and Company’s services and products, your violation of the Terms or your violation of any rights of third parties, or your violation of the applicable law. The may assume the exclusive defence and you shall cooperate with the Company in asserting any available defences.\n    9. TERMINATION AND ACCESS RESTRICTION \nThe Company may terminate your access and account to the Site and its related services or any part at any time, without notice, in case of your violation of the Terms.\n    10. MISCELLANEOUS\nThe governing law of the Terms shall be the substantive laws of the country where the Company is set up, except the conflict of laws rules. You shall not use the Site in jurisdictions that do not give effect to all provisions of the Terms.\nNo joint venture, partnership, employment, or agency relationship shall be implied between you and the Company as a result of the Terms or use of the Site.\nNothing in the Terms shall be a derogation of the Company's right to comply with governmental, court, police, and law enforcement requests or requirements regarding your enjoyment of the Site.\nIf any part of the Terms is determined to be void or unenforceable in accordance with applicable law then the void or unenforceable clauses will be deemed superseded by valid and enforceable clauses shall be similar to the original version of the Terms and other parts and sections of the Terms shall be applicable to you and the Company.\nThe Terms constitute the entire agreement between you and the Company regarding the enjoyment of the Site and the Terms supersede all prior or communications and offers, whether electronic, oral or written, between you and the Company.\nThe Company and its affiliates shall not be liable for a failure or delay to fulfil its obligations where the failure or delay results from any cause beyond Company's reasonable control, including technical failures, natural disasters, blockages, embargoes, riots, acts, regulation, legislation, or orders of government, terroristic acts, war, or any other force outside of Company's control.\nIn case of controversies, demands, claims, disputes, or causes of action between the Company and you relating to the Site or other related issues, or the Terms, you and the Company agree to attempt to resolve such controversies, demands, claims, disputes, or causes of action by good faith negotiation, and in case of failure of such negotiation, exclusively through the courts of the country where the Company is set up.\n    11. Complaints\nWe are committed to resolve any complaints about our collection or use of your personal data. If you would like to make a complaint regarding this  Terms or our practices in relation to your personal data, please contact us at: [INSERT CONTACT DETAILS FOR COMPANY'S INTERNAL COMPLAINTS MECHANISM]. We will reply to your complaint as soon as we can and in any event, within 30 days. We hope to resolve any complaint brought to our attention, however if you feel that your complaint has not been adequately resolved, you reserve the right to contact your local data protection supervisory authority\n    12. Contact Information\nWe welcome your comments or questions about this  Terms. You may contact us in writing at [Insert email address] or [Insert postal address].",
            'text_color'        => "#4A58A0",
            'button_color'      => "#4A58A0",
            'button_text_color' => "#FFF",
            'email'             => 'gdpr@test.com',
            'pp_link'           => site_url() . '/privacy-policy',
            'is_required'       => 0,
            'countries'         => json_encode([
                "BE", "BG", "CZ", "DK", "DE", "EE", "IE", "EL", "ES", "FR", "HR", "IT", "CY",
                "LV", "LT", "LU", "HU", "MT", "NL", "AT", "PL", "PT", "RO", "SI", "SK", "FI",
                "SE", "GB"
            ]) //EU countries
        ], []);
    }

    public function getAllPopups()
    {
        global $wpdb;

        $table_name = $wpdb->prefix . $this->pluginOptions->popupsTable;
        $popups = $wpdb->get_results(
            "
            SELECT * FROM $table_name
            WHERE is_deleted = 0
            "
        );

        $table_name = $wpdb->prefix . $this->pluginOptions->popupPagesTable;
        $popupPages = $wpdb->get_results(
            "
            SELECT * FROM $table_name
            "
        );

        foreach ($popups as &$popup) {
            $popup->pages = [];
            $popup->countries = json_decode($popup->countries);

            foreach ($popupPages as $i => &$pp) {
                if ($pp->popup_id == $popup->id) {
                    $popup->pages[] = $pp->page_id;
                    unset($popupPages[$i]);
                }
            }
        }

        return $popups;
    }

    public function getAllPages()
    {
        global $wpdb;

        $table_name = $wpdb->prefix . 'posts';
        $pages = $wpdb->get_results(
            "
            SELECT ID, post_title FROM $table_name
            WHERE post_status = 'publish' AND post_type='page'
            "
        );

        return $pages;
    }

    public function insertPopups($popups)
    {
        global $wpdb;

        foreach ($popups as $popup) {

            $popupsTableName = $wpdb->prefix . $this->pluginOptions->popupsTable;
            $wpdb->query($wpdb->prepare(
                "
		        INSERT INTO $popupsTableName (title, internal_name, description, terms, text_color, color, button_color, button_text_color, redirect_link, countries, is_required, submit_button_text, cancel_button_text, email, preference_link, pp_link)  
                VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %d, %s, %s, %s, %s, %s)
            ", $popup['title'], $popup['internal_name'], $popup['description'], $popup['terms'], $popup['text_color'], $popup['color'], $popup['button_color'], $popup['button_text_color'], $popup['redirect_link'], $popup['countries'], $popup['is_required'], $popup['submit_button_text'], $popup['cancel_button_text'], $popup['email'], $popup['preference_link'], $popup['pp_link']
            ));
            $popupID = $wpdb->insert_id;

            $popupPagesTableName = $wpdb->prefix . $this->pluginOptions->popupPagesTable;
            if (isset($popup['pages'])) {
                foreach ($popup['pages'] as $pageID)
                    $wpdb->query($wpdb->prepare(
                        "
                            INSERT INTO $popupPagesTableName (popup_id, page_id) 
                            VALUES (%d, %d)
                        ", $popupID, $pageID
                    ));
            }
        }
    }

    public function updatePopups($popups)
    {
        global $wpdb;

        foreach ($popups as $popup) {
            $popupPagesTableName = $wpdb->prefix . $this->pluginOptions->popupPagesTable;
            $wpdb->query($wpdb->prepare(
                "
		        DELETE FROM $popupPagesTableName   
                WHERE popup_id = %d
            ", $popup['id']
            ));

            if (isset($popup['pages'])) {
                foreach ($popup['pages'] as $pageID)
                    $wpdb->query($wpdb->prepare(
                        "
                            INSERT INTO $popupPagesTableName (popup_id, page_id) 
                            VALUES (%d, %d)
                        ", $popup['id'], $pageID
                    ));
            }

            $table_name = $wpdb->prefix . $this->pluginOptions->popupsTable;
            $wpdb->query($wpdb->prepare(
                "
		        UPDATE $table_name   
                SET title = %s, internal_name = %s, description = %s, terms = %s, text_color = %s, color = %s, button_color = %s, button_text_color = %s, redirect_link = %s, countries = %s, is_required = %s, submit_button_text = %s, cancel_button_text = %s, email = %s, preference_link = %s, pp_link = %s
                WHERE id = ( %d )
            ", $popup['title'], $popup['internal_name'], $popup['description'], $popup['terms'], $popup['text_color'], $popup['color'], $popup['button_color'], $popup['button_text_color'], $popup['redirect_link'], $popup['countries'], $popup['is_required'], $popup['submit_button_text'], $popup['cancel_button_text'], $popup['email'], $popup['preference_link'], $popup['pp_link'], $popup['id']
            ));
        }
    }

    public function markPopupsAsDeleted($ids)
    {
        if (!$ids)
            return;

        $idsText = implode(",", $ids);

        global $wpdb;
        $popupPagesTableName = $wpdb->prefix . $this->pluginOptions->popupPagesTable;
        foreach ($ids as $popupID)
            $wpdb->query($wpdb->prepare(
                "
                    DELETE FROM $popupPagesTableName   
                    WHERE popup_id = %d
                ", $popupID
            ));

        $table_name = $wpdb->prefix . $this->pluginOptions->popupsTable;
        $wpdb->query($wpdb->prepare(
            "
		        UPDATE $table_name   
                SET is_deleted = 1
                WHERE id IN ( %s )
            ",
            $idsText
        ));
    }

    public function getPopupForPage($pageID)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . $this->pluginOptions->popupPagesTable;
        $r = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name WHERE page_id = %d", $pageID));

        if (!$r)
            return NULL;

        $table_name = $wpdb->prefix . $this->pluginOptions->popupsTable;
        $popup = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name WHERE id = %d", $r->popup_id));
        $popup->countries = json_decode($popup->countries);

        return $popup;
    }

    public function paginateUsers($page = 0, $perPage = 20, $searchTerm = null)
    {
        global $wpdb;
        $table_name_users = $wpdb->prefix . $this->pluginOptions->usersTable;
        $table_name_popups = $wpdb->prefix . $this->pluginOptions->popupsTable;

        $offset = $page * $perPage;

        if ($searchTerm) {
            $searchTerm = "%" .  $searchTerm . "%";
            return $wpdb->get_results($wpdb->prepare(
                "
                    SELECT $table_name_users.*, $table_name_popups.id as pid, $table_name_popups.internal_name FROM $table_name_users
                    LEFT JOIN $table_name_popups on $table_name_users.popup_id = $table_name_popups.id
                    WHERE $table_name_users.first_name LIKE %s
                    OR $table_name_users.last_name LIKE %s
                    OR CONCAT($table_name_users.first_name, ' ', $table_name_users.last_name) LIKE %s
                    OR $table_name_users.email LIKE %s
                    ORDER BY $table_name_users.time DESC
                    LIMIT %d
                    OFFSET %d
                    ", $searchTerm, $searchTerm, $searchTerm, $searchTerm, $perPage, $offset
            ));
        } else {
            return $wpdb->get_results($wpdb->prepare(
                "
                    SELECT $table_name_users.*, $table_name_popups.id as pid, $table_name_popups.internal_name FROM $table_name_users
                    LEFT JOIN $table_name_popups on $table_name_users.popup_id = $table_name_popups.id
                    ORDER BY $table_name_users.time DESC
                    LIMIT %d
                    OFFSET %d
                    ", $perPage, $offset
            ));
        }
    }

    public function getAllUsers($consentOnly)
    {
        global $wpdb;
        $table_name_users = $wpdb->prefix . $this->pluginOptions->usersTable;
        $table_name_popups = $wpdb->prefix . $this->pluginOptions->popupsTable;
        $query = "
                SELECT $table_name_users.first_name, $table_name_users.last_name, $table_name_users.email, $table_name_users.consent, $table_name_users.time, $table_name_popups.internal_name FROM $table_name_users
                LEFT JOIN $table_name_popups on $table_name_users.popup_id = $table_name_popups.id 
                
        ";

        if ($consentOnly) {
            $query .= " WHERE $table_name_users.consent = 1 ";
        }

        $query .= " ORDER BY $table_name_users.time DESC";

        return $wpdb->get_results($query, ARRAY_A);
    }

    public function countTotalUsers()
    {
        global $wpdb;
        $table_name_users = $wpdb->prefix . $this->pluginOptions->usersTable;

        return $wpdb->get_var("SELECT COUNT(*) FROM $table_name_users");
    }

    public function insertNewUser($popupID, $firstName, $lastName, $email, $consent = 0)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . $this->pluginOptions->usersTable;
        $wpdb->query($wpdb->prepare(
            "
		        INSERT INTO $table_name (popup_id, first_name, last_name, email, consent)
                VALUES (%d, %s, %s, %s, %d)
            ", $popupID, $firstName, $lastName, $email, $consent
        ));
    }

    public function deleteUser($userID)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . $this->pluginOptions->usersTable;
        $wpdb->query($wpdb->prepare(
            "
		        DELETE FROM $table_name 
                WHERE id = %d
            ", $userID
        ));
    }
}