var app = new Vue({
    el: '#gdprapp',
    components: {
        'chrome-picker': window.VueColor.Chrome
    },
    data: function () {
        return {
            popups: [],
            saving: false,
            saved: false,
            errors: [],
            deleted: [],
        }
    },
    mounted: function () {
        this.getPopups()
    },
    methods: {
        getPopups: function () {
            var vm = this;
            jQuery.ajax( {
                url: gdprApiSettings.plugin_root + '/popups',
                method: 'get',
                beforeSend: function ( xhr ) {
                    xhr.setRequestHeader( 'X-WP-Nonce', gdprApiSettings.nonce );
                }
            } ).done( function ( response ) {
                console.log( response );
                vm.popups = response;
            } );
        },
        selectEU: function (i) {
            this.popups[i].countries = [
                "BE", "BG", "CZ", "DK", "DE", "EE", "IE", "EL", "ES", "FR", "HR", "IT", "CY",
                "LV", "LT", "LU", "HU", "MT", "NL", "AT", "PL", "PT", "RO", "SI", "SK", "FI",
                "SE"
            ];
        },
        selectEUUK: function (i) {
            this.popups[i].countries = [
                "BE", "BG", "CZ", "DK", "DE", "EE", "IE", "EL", "ES", "FR", "HR", "IT", "CY",
                "LV", "LT", "LU", "HU", "MT", "NL", "AT", "PL", "PT", "RO", "SI", "SK", "FI",
                "SE", "GB"
            ];
        },
        selectAll: function (i) {
            this.popups[i].countries = gdprApiSettings.countries;
        },
        selectNone: function (i) {
            this.popups[i].countries = [];
        },
        removePopup: function (i) {
            this.deleted.push(this.popups[i]);
            this.popups.splice(i, 1);
        },
        addNewPopup: function () {
            this.popups.push({text_color: '#FFFFFF', color: "#1A8DFC", button_color: "#1A8DFC", button_text_color: "#FFFFFF", is_required: 0, countries: [], pages: [], title: "Title"})
        },
        save: function () {
            var vm = this;
            vm.saving = true;
            vm.saved = false;
            vm.errors = [];
            vm.errors = this.validate();

            if (vm.errors.length) {
                vm.saving = false;
                return;
            }

            jQuery.ajax( {
                url: gdprApiSettings.plugin_root + '/popups',
                method: 'POST',
                beforeSend: function ( xhr ) {
                    xhr.setRequestHeader( 'X-WP-Nonce', gdprApiSettings.nonce );
                },
                data: {popups: vm.popups, deleted: vm.deleted}
            } ).done( function ( response ) {
                vm.saving = false;
                vm.saved = true;
                vm.deleted = [];
                vm.refreshAfterSave();
            } ).error(function (response) {
                vm.saving = false;
                vm.errors.push(['Unknown server error. Popups not saved']);
            });
        },

        validate: function () {
            var errors = [];
            var usedCountries = [];
            this.popups.forEach(function (p, i) {
                if (!p.title)
                    errors.push("Popup " + (i+1) + " has no title.")

                if (!p.terms)
                    errors.push("Popup " + (i+1) + " has no terms of service.")

                usedCountries.push(p.countries);
            })

            var commonCountries = [];
            if (this.popups.length > 1)
                commonCountries = this.intersectArrays(usedCountries);

            if (commonCountries.length)
                errors.push("More than one popup assigned to same country. Only 1 popup per country is possible.")

            return errors;
        },
        intersectArrays: function(arrays) {
            var min = 1000; var arg = 0; var index = 0;
            var common = [];
            for (var i=0; i<arrays.length; i++){
                if(arrays[i].length < min){
                    min = arrays[i].length;
                    arg = i;
                }
            }
            for (var i=0; i<arrays[arg].length; i++){
                for (var j=0; j<arrays.length; j++){
                    if(j!=arg && arrays[j].indexOf(arrays[arg][i]) != -1){
                        index++;
                    }
                }
                if(index == arrays.length-1){
                    common.push(arrays[arg][i]);
                }
                index = 0;
            }
            return common;
        },
        refreshAfterSave: function()
        {
            var url = window.location.href;
            if(url.indexOf("&popupssaved") === -1) {
                url += "&popupssaved=1";
            }
            window.location.replace(url);
        }
    }
})