<?php

namespace GDPRGeoPopup\src;

use GDPRGeoPopup\database\DB;
use GDPRGeoPopup\PluginOptions;
use WP_REST_Request;
use WP_REST_Response;

class AdminAPI
{
    /**
     * @var PluginOptions
     */
    protected $pluginOptions;

    /**
     * @var DB
     */
    protected $db;

    public function __construct($pluginOptions, $db)
    {
        $this->pluginOptions = $pluginOptions;
        $this->db = $db;
    }

    public function init()
    {
        add_action( 'rest_api_init', function () {
            register_rest_route( $this->pluginOptions->safename, '/popups', array(
                'methods' => 'GET',
                'callback' => [$this, 'getPopups'],
            ) );

            register_rest_route( $this->pluginOptions->safename, '/popups', array(
                'methods' => 'POST',
                'callback' => [$this, 'postPopups'],
            ) );
        } );
    }

    public function getPopups(WP_REST_Request $request)
    {
        if (!current_user_can('manage_options'))
            return;

        $data = $this->db->getAllPopups();

        $response = new WP_REST_Response($data);

        return $response;
    }

    public function postPopups(WP_REST_Request $request)
    {
        if (!current_user_can('manage_options'))
            return;

        $data = $request->get_body_params();

        //delete ones which were removed
        if (isset($data['deleted']) && count($data['deleted'])) {
            $toDeleteIDs = [];
            foreach ($data['deleted'] as $d) {
                if (isset($d['id']) && $d['id'])
                    $toDeleteIDs[] = $d['id'];
            }

            $this->db->markPopupsAsDeleted($toDeleteIDs);
        }

        $toInsert = [];
        $toUpdate = [];

        foreach ($data['popups'] as &$popup)
        {
            $popup = $this->preparePopup($popup);

            if (isset($popup['id']) && $popup['id'])
                $toUpdate[] = $popup;
            else
                $toInsert[] = $popup;
        }

        $this->db->updatePopups($toUpdate);
        $this->db->insertPopups($toInsert);

        return $data;
    }

    private function preparePopup($popup)
    {
        $popup['title'] = isset($popup['title']) ? $popup['title'] : '';
        $popup['internal_name'] = isset($popup['internal_name']) && $popup['internal_name'] ? $popup['internal_name'] : null;
        $popup['description'] = isset($popup['description']) && $popup['description'] ? $popup['description'] : null;
        $popup['terms'] = isset($popup['terms']) && $popup['terms'] ? $popup['terms'] : null;

        if (isset($popup['text_color']) && $popup['text_color']) {
            if (isset($popup['text_color']['hex']))
                $popup['text_color'] = $popup['text_color']['hex'];
        } else {
            $popup['text_color'] = "#000000";
        }

        if (isset($popup['color']) && $popup['color']) {
            if (isset($popup['color']['hex']))
                $popup['color'] = $popup['color']['hex'];
        } else {
            $popup['color'] = "#000000";
        }

        if (isset($popup['button_color']) && $popup['button_color']) {
            if (isset($popup['button_color']['hex']))
                $popup['button_color'] = $popup['button_color']['hex'];
        } else {
            $popup['button_color'] = "#000000";
        }

        if (isset($popup['button_text_color']) && $popup['button_text_color']) {
            if (isset($popup['button_text_color']['hex']))
                $popup['button_text_color'] = $popup['button_text_color']['hex'];
        } else {
            $popup['button_text_color'] = "#000000";
        }

        $popup['redirect_link'] = isset($popup['redirect_link']) && $popup['redirect_link'] ? $popup['redirect_link'] : NULL;
        if  ($popup['redirect_link'] ) {
            if ($ret = parse_url($popup['redirect_link']))
                if ( !isset($ret["scheme"]) )
                {
                    $popup['redirect_link'] = "http://" . $popup['redirect_link'];
                }
        }

        $popup['preference_link'] = isset($popup['preference_link']) && $popup['preference_link'] ? $popup['preference_link'] : NULL;
        if  ($popup['preference_link'] ) {
            if ($ret = parse_url($popup['preference_link']))
                if ( !isset($ret["scheme"]) )
                {
                    $popup['preference_link'] = "http://" . $popup['preference_link'];
                }
        }

        $popup['pp_link'] = isset($popup['pp_link']) && $popup['pp_link'] ? $popup['pp_link'] : NULL;
        if  ($popup['pp_link'] ) {
            if ($ret = parse_url($popup['pp_link']))
                if ( !isset($ret["scheme"]) )
                {
                    $popup['pp_link'] = "http://" . $popup['pp_link'];
                }
        }


        $popup['submit_button_text'] = isset($popup['submit_button_text']) && $popup['submit_button_text'] ? $popup['submit_button_text'] : 'Submit';
        $popup['cancel_button_text'] = isset($popup['cancel_button_text']) && $popup['cancel_button_text'] ? $popup['cancel_button_text'] : 'Cancel';

        $popup['email'] = isset($popup['email']) && $popup['email'] ? $popup['email'] : NULL;

        $popup['countries'] = isset($popup['countries']) && $popup['countries'] ? $popup['countries'] : [];
        $popup['countries'] = json_encode($popup['countries']);
        $popup['is_required'] = isset($popup['is_required']) && $popup['is_required'] ? 1 : 0;

        return $popup;
    }
}