<?php

namespace GDPRGeoPopup\src;

use GDPRGeoPopup\database\DB;
use GDPRGeoPopup\PluginOptions;

class AdminConfigPage
{
    /**
     * @var PluginOptions
     */
    protected $pluginOptions;

    /**
     * @var DB
     */
    protected $db;

    public function __construct($pluginOptions, $db)
    {
        $this->pluginOptions = $pluginOptions;
        $this->db = $db;
    }

    public function init()
    {
        add_action('admin_menu', [$this, 'addPage']);
        add_action( 'admin_enqueue_scripts', [$this, 'enqueueScripts']);
    }

    public function addPage()
    {
        add_menu_page(
            $this->pluginOptions->name,
            $this->pluginOptions->name,
            'manage_options',
            $this->pluginOptions->menuslug,
            [$this, 'view']
        );
    }

    public function view()
    {
        if (!current_user_can('manage_options')) {
            return;
        }

        $countries = require 'countries.php';
        $pages = $this->db->getAllPages();
        include $this->pluginOptions->pluginPath . '/views/config-page.php';
    }

    public function enqueueScripts($hook) {
        if ($hook != 'toplevel_page_'. $this->pluginOptions->menuslug)
            return;

        wp_enqueue_style('gdpr-css', plugins_url($this->pluginOptions->safename . '/css/gdpr-geopopup-admin.css'));

        wp_enqueue_script('jquery');
        wp_enqueue_script('vue', plugins_url($this->pluginOptions->safename . '/js/vue.min.js'));
        wp_enqueue_script('vue-color', plugins_url($this->pluginOptions->safename . '/js/vue-color.min.js'));
        wp_enqueue_script('gdpr-wp-api', plugins_url($this->pluginOptions->safename . '/js/gdpr-geopopup-config.js'), [], false, true);


        $countries = require 'countries.php';
        $cs = [];
        foreach ($countries as $code => $country) {
            $cs[] = $code;
        }
        wp_localize_script( 'gdpr-wp-api', 'gdprApiSettings', [
            'root' => esc_url_raw( rest_url() ),
            'plugin_root' => esc_url_raw( rest_url() ) . $this->pluginOptions->safename,
            'nonce' => wp_create_nonce( 'wp_rest' ),
            'countries' =>  $cs
        ]);
    }
}