<?php

namespace GDPRGeoPopup\src;

use GDPRGeoPopup\database\DB;
use GDPRGeoPopup\PluginOptions;

class AdminUsersPage
{
    /**
     * @var PluginOptions
     */
    protected $pluginOptions;

    /**
     * @var DB
     */
    protected $db;

    public function __construct($pluginOptions, $db)
    {
        $this->pluginOptions = $pluginOptions;
        $this->db = $db;
    }

    public function init()
    {
        add_action('admin_menu', [$this, 'addPage']);

        add_action('admin_post_gdprpopup_download_users', [$this, 'downloadUsers']  );
        add_action('admin_post_gdprpopup_download_users_consent', [$this, 'downloadUsersConsent']  );
    }

    public function addPage()
    {
        add_submenu_page(
            $this->pluginOptions->menuslug,
            'Users',
            'Users',
            'manage_options',
            $this->pluginOptions->menuslug . '-users',
            [$this, 'view']
        );
    }

    public function view()
    {
        if (!current_user_can('manage_options')) {
            return;
        }

        $currentPage  = isset($_GET['paged']) ? $_GET['paged'] : 0;
        $perPage = 20;

        $searchTerm = null;
        if (isset($_POST['search'])) {
            $searchTerm = $_POST['search'];
        }

        if (isset($_POST['uid'])) {
            $this->db->deleteUser($_POST['uid']);
        }

        $users = $this->db->paginateUsers($currentPage, $perPage, $searchTerm);
        $totalUsers = $this->db->countTotalUsers();

        $nrOfPages = ceil($totalUsers / $perPage);

        if (!current_user_can('manage_options')) {
            return;
        }

        include $this->pluginOptions->pluginPath . '/views/users-page.php';

    }

    public function downloadUsers()
    {
        if (!current_user_can('manage_options')) {
            return;
        }

        $this->downloadCSV(false);
    }

    public function downloadUsersConsent()
    {
        if (!current_user_can('manage_options')) {
            return;
        }

        $this->downloadCSV(true);
    }

    private function downloadCSV($consentOnly)
    {
        header('Content-Type: application/csv');
        header('Content-Disposition: attachment; filename="popupusers.csv";');

        $users = $this->db->getAllUsers($consentOnly);

        $f = fopen('php://output', 'w');
        fputcsv($f, ['FIRST NAME', 'LAST NAME', 'EMAIL', 'CONSENTED?', 'TIME', 'POPUP NAME'], ";");

        foreach ($users as $user) {
            $user['consent'] = $user['consent'] ? "YES" : "NO";
            fputcsv($f, $user, ";");
        }
    }
}