<?php

namespace GDPRGeoPopup\src;

use GDPRGeoPopup\database\DB;
use GDPRGeoPopup\PluginOptions;
use MaxMind\Db\Reader;

class FrontendPopup
{
    /**
     * @var PluginOptions
     */
    protected $pluginOptions;

    /**
     * @var DB
     */
    protected $db;

    public function __construct($pluginOptions, $db)
    {
        $this->pluginOptions = $pluginOptions;
        $this->db = $db;
    }

    public function init()
    {
        add_action( 'wp_footer', [$this, 'view'] );

        add_action( 'admin_post_gdprpopup_store_data', [$this, 'popupPOSTForm']  );
        add_action( 'admin_post_nopriv_gdprpopup_store_data', [$this, 'popupPOSTForm']  );

        add_action( 'wp_enqueue_scripts', [$this, 'enqueueScriptsAndStyles']  );
    }

    public function popupPOSTForm()
    {
        if (!$_POST) {
            wp_redirect(home_url());
            return;
        }

        if (!isset($_POST['pid']) ||
            !isset($_POST['first_name']) ||
            !isset($_POST['last_name']) ||
            !isset($_POST['email']))
        {
            wp_redirect(home_url());
            return;
        }

        $this->db->insertNewUser(trim($_POST['pid']), trim($_POST['first_name']), trim($_POST['last_name']), trim($_POST['email']), isset($_POST['gdpr-geopopup-consent']) ? 1 : 0);

        wp_redirect(isset($_POST['continueurl']) ? $_POST['continueurl'] : home_url());
    }

    public function view()
    {
        if (!is_page())
            return;

        $popup = $this->db->getPopupForPage(get_the_ID());

        if (!$popup)
            return;

        $ip = $_SERVER['REMOTE_ADDR'];
        $reader = new Reader($this->pluginOptions->pluginPath . '/database/GeoLite2-Country.mmdb' );
        $countryData = $reader->get($ip);

        if (!isset($countryData['country']['iso_code']))
            return;

        $countryISO = $countryData['country']['iso_code'];
        $countryName = $countryData['country']['names']['en'];
        $showInCurrentCountry = in_array($countryISO, $popup->countries);

        if (!$showInCurrentCountry)
            return;

        $popup->description = str_replace('%COUNTRY%', $countryName, $popup->description);

        include $this->pluginOptions->pluginPath . '/views/popup.php';
    }

    public function enqueueScriptsAndStyles()
    {
        wp_enqueue_script('jquery');
        wp_enqueue_script('vue-color', plugins_url($this->pluginOptions->safename . '/js/gdpr-geopopup.js'));
        wp_enqueue_style('montserrat', 'https://fonts.googleapis.com/css?family=Roboto:400,700');
        wp_enqueue_style('gdpr-geo-popup', plugins_url($this->pluginOptions->safename . '/css/gdpr-geopopup.css'));
    }
}
